/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/*
 * MapRegion.cpp
 *
 *  Created on: Oct 13, 2010
 *      Author: ltwardon
 */
#include "MapRegion.h"
#include <cfloat>

using namespace std;
using namespace airobots;

namespace airobots {

const std::string MapRegion::POLYGONE_TYPE = "polygon";
const std::string MapRegion::Rectangle_TYPE = "rectangle";

MapRegion::MapRegion() :
		type(""), label(""), group(""), height(0.0), viewPointSet(false) {

}

MapRegion::MapRegion(std::string type) :
		type(type), label(""), group(""), height(0.0), viewPointSet(false) {

}

MapRegion::MapRegion(std::string type, std::string label) :
		type(type), label(label), group(""), height(0.0), viewPointSet(false) {

}

MapRegion::MapRegion(std::string type, std::string label, std::string group) :
		type(type), label(label), group(group), height(0.0), viewPointSet(false) {

}

MapRegion::MapRegion(std::string type, std::string label, std::string group,
		list<MapPoint> points) :
		type(type), label(label), group(group), points(points), height(0.0), viewPointSet(
				false) {

}

MapRegion::MapRegion(std::string type, std::string label, std::string group,
		list<MapPoint> points, NavigationGoalData viewPoint) :
		type(type), label(label), group(group), points(points), viewPoint(
				viewPoint), height(0.0), viewPointSet(true) {

}

MapRegion::MapRegion(std::string type, std::string label, std::string group,
		list<MapPoint> points, NavigationGoalData viewPoint, double height,LengthUnit lU) :
		type(type), label(label), group(group), points(points), viewPoint(
				viewPoint),  viewPointSet(true) {
		this->height = UnitConverter::convert(height,lU,iLU);
}

void MapRegion::addPoint(double x, double y, LengthUnit lengthUnit) {
	MapPoint point(x, y, lengthUnit);
	points.push_back(point);
}

void MapRegion::addPoint(MapPoint point) {
	points.push_back(point);
}

std::string MapRegion::getType() const {
	return type;
}

void MapRegion::setType(std::string type) {
	this->type = type;
}

std::string MapRegion::getLabel() const {
	return label;
}

void MapRegion::setLabel(std::string label) {
	this->label = label;
}

std::string MapRegion::getGroup() const {
	return group;
}

void MapRegion::setGroup(std::string group) {
	this->group = group;
}

list<MapPoint> MapRegion::getPoints() const {
	return points;
}

void MapRegion::setPoints(list<MapPoint> points) {
	this->points = points;
}

NavigationGoalData MapRegion::getViewPoint() const {
	return viewPoint;
}

void MapRegion::setViewPoint(NavigationGoalData viewPoint) {
	this->viewPoint = viewPoint;
	viewPointSet = true;
}

double MapRegion::getHeight(LengthUnit lU) const {
	return UnitConverter::convert(height,iLU,lU);
}

void MapRegion::setHeight(double height,LengthUnit lU) {
	this->height = UnitConverter::convert(height,lU,iLU);
}

void MapRegion::removePoint(double x, double y) {

	list<MapPoint>::iterator i;
	for (i = points.begin(); i != points.end();) {
		if (i->getX(METER) == x && i->getY(METER) == y) {
			i = points.erase(i);
		} else {
			++i;
		}
	}

}

int MapRegion::getPointCount() const {
	return (int) points.size();
}

PositionData MapRegion::getPositionData() const {

	if (viewPointSet) {
		return viewPoint;
	} else {
		double min_x = DBL_MAX;
		double min_y = DBL_MAX;
		double max_x = DBL_MIN;
		double max_y = DBL_MIN;
		list<MapPoint>::const_iterator i;
		for (i = points.begin(); i != points.end(); ++i) {
			if (i->getX(METER) < min_x)
				min_x = i->getX(METER);
			if (i->getY(METER) < min_y)
				min_y = i->getY(METER);
			if (i->getX(METER) > max_x)
				max_x = i->getX(METER);
			if (i->getY(METER) > max_y)
				max_y = i->getY(METER);
		}
		PositionData p((max_x + min_x) / 2, (max_y + min_y) / 2, 0,METER,RADIAN);
		return p;
	}

}

bool MapRegion::isViewPointSet() const {
	return viewPointSet;
}

bool MapRegion::contains(double x, double y) const {

	int npoints = (int) points.size();

	if (npoints <= 2) {
		return false;
	}
	int hits = 0;

	double lastx = points.back().getX(METER);
	double lasty = points.back().getY(METER);
	double curx, cury;

	// Walk the edges of the polygon
	for (list<MapPoint>::const_iterator i = points.begin(); i != points.end();
			lastx = curx, lasty = cury, i++) {
		curx = i->getX(METER);
		cury = i->getY(METER);

		if (cury == lasty) {
			continue;
		}

		double leftx;
		if (curx < lastx) {
			if (x >= lastx) {
				continue;
			}
			leftx = curx;
		} else {
			if (x >= curx) {
				continue;
			}
			leftx = lastx;
		}

		double test1, test2;
		if (cury < lasty) {
			if (y < cury || y >= lasty) {
				continue;
			}
			if (x < leftx) {
				hits++;
				continue;
			}
			test1 = x - curx;
			test2 = y - cury;
		} else {
			if (y < lasty || y >= cury) {
				continue;
			}
			if (x < leftx) {
				hits++;
				continue;
			}
			test1 = x - lastx;
			test2 = y - lasty;
		}

		if (test1 < (test2 / (lasty - cury) * (lastx - curx))) {
			hits++;
		}
	}

	return ((hits & 1) != 0);

}

}
