/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data.speechrec;

import java.util.LinkedList;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;

import org.apache.log4j.Logger;

import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;

/**
 * Stores an specific gpsrAction for tobi. it contains the name and a list of
 * parameters. e.g. name = goTo params: kitchen.
 * 
 * @author hterhors
 */
public class GPSRAction extends Type {

	private static Logger logger = Logger.getLogger(GPSRAction.class);

	private String name;
	private LinkedList<GPSRParameter> params = new LinkedList<GPSRParameter>();

	/** Constructor. */
	public GPSRAction() {
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public LinkedList<GPSRParameter> getParams() {
		return params;
	}

	public void addParam(GPSRParameter param) {
		this.params.add(param);
	}

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof GPSRAction) {
			GPSRAction a = (GPSRAction) obj;

			return name.equals(a.getName()) && params.equals(a.getParams());
		}
		return false;
	}

	public static String getBaseTag() {
		return "GPSRACTION";
	}

	/** @{inheritDoc */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);
		Element actionElement = new Element("ACTION");
		actionElement.addAttribute(new Attribute("name", name));
		for (GPSRParameter p : params) {
			Element paramElement = new Element("PARAMETER");
			paramElement.addAttribute(new Attribute("name", p.getName()));
			paramElement.addAttribute(new Attribute("type", p.getType()));
			actionElement.appendChild(paramElement);
		}
		parent.appendChild(actionElement);
	}

	/** @{inheritDoc */
	public static void fromElement(Element element, GPSRAction action) {

		Type.fromElement(element, action);

		Nodes nodes = element.query("ACTION");
		int nrNodes = nodes.size();
		for (int i = 0; i < nrNodes; i++) {
			Node actionNode = nodes.get(i);
			if (actionNode instanceof Element) {
				Element actionElem = (Element) actionNode;
				String name = ElementParser.getAttributeValue(actionElem,
						"name");
				action.setName(name);

				Nodes paramNodes = actionElem.query("PARAMETER");
				int nrParamNodes = paramNodes.size();
				for (int j = 0; j < nrParamNodes; j++) {
					Node paramNode = paramNodes.get(j);
					if (paramNode instanceof Element) {
						Element paramElement = (Element) paramNode;
						GPSRParameter param = action.new GPSRParameter(
								ElementParser.getAttributeValue(paramElement,
										"name"),
								ElementParser.getAttributeValue(paramElement,
										"type"));
						action.addParam(param);
					}
				}
			}
		}
	}

	public class GPSRParameter {
		String name;
		String type;

		/**
		 * 
		 * @param name
		 *            the name of the parameter
		 * @param type
		 *            could be <b>'location'</b> or <b>'object'</b>
		 */
		public GPSRParameter(String name, String type) {
			this.name = name;
			this.type = type;
		}

		public String getName() {
			return name;
		}

		public String getType() {
			return type;
		}

		@Override
		public boolean equals(Object obj) {
			if (obj instanceof GPSRParameter) {
				GPSRParameter a = (GPSRParameter) obj;

				return name.equals(a.getName()) && type.equals(a.getType());
			}
			return false;
		}

	}

}
